@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 ###########################################################################
 #    OCRFeeder - The complete OCR suite
 #    Copyright (C) 2009-2013 Joaquim Rocha
@@ -18,20 +16,21 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
-from boxesarea import BoxesArea
-from dataHolder import DataBox, PageData, TEXT_TYPE, IMAGE_TYPE
+from .boxesarea import BoxesArea
+from .dataHolder import DataBox, PageData, TEXT_TYPE, IMAGE_TYPE
 from ocrfeeder.feeder.documentGeneration import OdtGenerator, HtmlGenerator, PlaintextGenerator, PdfGenerator
 from ocrfeeder.feeder.imageManipulation import *
 from ocrfeeder.feeder.layoutAnalysis import *
-from project import ProjectSaver, ProjectLoader
+from .project import ProjectSaver, ProjectLoader
 from ocrfeeder.util import graphics, ALIGN_LEFT, ALIGN_RIGHT, ALIGN_CENTER, \
      ALIGN_FILL, PAPER_SIZES
-from ocrfeeder.util.lib import debug, getNonExistingFileName, unpaperImage
+from ocrfeeder.util.lib import getNonExistingFileName, unpaperImage
+from ocrfeeder.util.log import debug, warning
 from ocrfeeder.util.configuration import ConfigurationManager
 from ocrfeeder.util import constants
 from ocrfeeder.util.asyncworker import AsyncItem
-from widgetPresenter import BoxEditor, PagesToExportDialog, FileDialog, \
-    PageSizeDialog, WarningDialog, UnpaperDialog, \
+from .widgetPresenter import BoxEditor, PagesToExportDialog, FileDialog, \
+    PageSizeDialog, UnpaperDialog, \
     QueuedEventsProgressDialog, SpellCheckerDialog
 import gettext
 from gi.repository import Gtk, GObject, Gdk, GdkPixbuf, Pango
@@ -41,10 +40,11 @@ import threading
 import sys
 _ = gettext.gettext
 
-class ImageReviewer(Gtk.HPaned):
+class ImageReviewer(Gtk.Paned):
 
     def __init__(self, main_window, page_data, ocr_engines):
-        super(ImageReviewer, self).__init__()
+        super(ImageReviewer, self).__init__(orientation=Gtk.Orientation.HORIZONTAL)
+        self.set_wide_handle(True)
         self.configuration_manager = ConfigurationManager()
         self.main_window = main_window
         self.path_to_image = page_data.image_path
@@ -59,7 +59,6 @@ class ImageReviewer(Gtk.HPaned):
         self.selectable_boxes_area.connect('deselected_box',
                                            self.deselectedBoxCb)
         self.image_pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.path_to_image)
-        self.set_position(500)
         self.show()
         self.ocr_engines = ocr_engines
         self.editor = Editor(self.image_pixbuf, self.ocr_engines, self)
@@ -74,7 +73,7 @@ class ImageReviewer(Gtk.HPaned):
         selectable_boxes_scrolled_window.show()
 
         self.pack1(selectable_boxes_scrolled_window, True, False)
-        self.pack2(self.editor.box_editor, True, False)
+        self.pack2(self.editor.box_editor, False, False)
         self.page = page_data
         self.updatePageData(self.page)
         selectable_boxes_scrolled_window.connect_after("size-allocate", self.zoomFitCb)
@@ -119,7 +118,7 @@ class ImageReviewer(Gtk.HPaned):
 
     def removedBox(self, widget, box):
         self.updateMainWindow()
-        if not self.boxes_dict.has_key(box):
+        if box not in self.boxes_dict:
             return False
         del self.boxes_dict[box]
         if self.editor.box == box:
@@ -245,10 +244,13 @@ class ImageReviewer(Gtk.HPaned):
             return
         try:
             self.image_pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.path_to_image)
-        except Exception, exception:
+        except Exception as exception:
             debug(exception.message)
             return
         self.selectable_boxes_area.setBackgroundImage(self.path_to_image)
+        self.editor.updateImage(self.image_pixbuf)
+        if self.editor.box:
+            self.editor.update(self.editor.box)
 
     def updateBoxesColors(self):
         self.editor.updateBoxColor()
@@ -316,6 +318,17 @@ class ImageReviewer_Controler:
         return image_reviewer
 
     def addImages(self, image_path_list):
+        paths = []
+        for path in image_path_list:
+            if os.path.exists(path):
+                paths.append(path)
+            else:
+                warning('Could not load image "%s": does not exist' % path)
+
+        if not paths:
+            return
+
+        image_path_list = paths
         item_list = []
         temp_dir = self.configuration_manager.TEMPORARY_FOLDER
         image_path_list = graphics.convertMultiImagesInList(image_path_list,
@@ -325,9 +338,9 @@ class ImageReviewer_Controler:
            not self.configuration_manager.unpaper_images_after_addition:
             for index in range(0, len(image_path_list)):
                 if not self.__addImage(image_path_list[index], index == 0):
-                    debug('Failed to load image "%s"' % image_path_list[index])
+                    debug('Failed to load image "%s"', image_path_list[index])
             return
-        dialog = QueuedEventsProgressDialog(self.main_window.window)
+        dialog = QueuedEventsProgressDialog(self.main_window)
         for index in range(0, item_list_length):
             image_path = image_path_list[index]
             item = AsyncItem(self.__imagePreProcessing,
@@ -337,12 +350,12 @@ class ImageReviewer_Controler:
                               index == item_list_length - 1,
                               index == 0))
             if item_list_length == 1:
-                item_info = (_('Preparing image'), _(u'Please wait…'))
+                item_info = (_('Preparing image'), _('Please wait…'))
             else:
                 item_info = (_('Preparing image %(current_index)s/%(total)s') % \
                              {'current_index': index + 1,
                               'total': item_list_length},
-                             _(u'Please wait…'))
+                             _('Please wait…'))
             item_list.append((item_info,item))
         dialog.setItemsList(item_list)
         dialog.run()
@@ -391,12 +404,12 @@ class ImageReviewer_Controler:
 
     def deskewCurrentImage(self, widget):
         reviewer = self.__getCurrentReviewer()
-        dialog = QueuedEventsProgressDialog(self.main_window.window)
+        dialog = QueuedEventsProgressDialog(self.main_window)
         item = AsyncItem(self.__deskewImage,
                          (reviewer.path_to_image,),
                          self.__deskewCurrentImageFinishedCb,
                          (dialog, reviewer))
-        item_info = (_('Deskewing image'), _(u'Please wait…'))
+        item_info = (_('Deskewing image'), _('Please wait…'))
         dialog.setItemsList([(item_info, item)])
         dialog.run()
 
@@ -466,7 +479,7 @@ class ImageReviewer_Controler:
         image_reviewer.performOcrForSelectedBoxes()
 
     def __confirmOveritePossibilityByRecognition(self):
-        confirm_recognition = Gtk.MessageDialog(self.main_window.window,
+        confirm_recognition = Gtk.MessageDialog(self.main_window,
                                                 message_type = Gtk.MessageType.QUESTION,
                                                 buttons = Gtk.ButtonsType.YES_NO,
                                                 flags = Gtk.DialogFlags.MODAL |
@@ -488,18 +501,18 @@ class ImageReviewer_Controler:
            self.__confirmOveritePossibilityByRecognition() != Gtk.ResponseType.YES:
                 return
         page = image_reviewer.page
-        dialog = QueuedEventsProgressDialog(self.main_window.window)
+        dialog = QueuedEventsProgressDialog(self.main_window)
         item = AsyncItem(self.__performRecognitionForPage,
                          (page,),
                          self.__performRecognitionForPageFinishedCb,
                          (dialog, page, [page]))
-        info = (_('Recognizing Page'), _(u'Please wait…'))
+        info = (_('Recognizing Page'), _('Please wait…'))
         dialog.setItemsList([(info, item)])
         dialog.run()
 
     def recognizeDocument(self):
         pages = self.pages_icon_view.getAllPages()
-        dialog = QueuedEventsProgressDialog(self.main_window.window)
+        dialog = QueuedEventsProgressDialog(self.main_window)
         items = []
         i = 1
         total = len(pages)
@@ -511,7 +524,7 @@ class ImageReviewer_Controler:
                              self.__performRecognitionForPageFinishedCb,
                              (dialog, page, pages))
             info = (_('Recognizing Document'),
-                    _(u'Recognizing page %(page_number)s/%(total_pages)s. Please wait…') % {'page_number': i,
+                    _('Recognizing page %(page_number)s/%(total_pages)s. Please wait…') % {'page_number': i,
                                                                                             'total_pages': total})
             items.append((info, item))
             i += 1
@@ -594,12 +607,19 @@ class ImageReviewer_Controler:
             if os.path.exists(file_name):
                 os.remove(file_name)
             document_generator = generator(file_name, **extra_args)
+            reviewer = self.__getCurrentReviewer()
             for page in pages:
+                # if the page is not the current reviewer, we need to
+                # update the box's image before exporting it
+                if page != reviewer.page:
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file(page.image_path)
+                    for box in page.data_boxes:
+                        box.updateImage(pixbuf)
                 document_generator.addPage(page)
             document_generator.save()
 
     def __askPdfFromScratch(self):
-        ask_pdf_type_dialog = Gtk.MessageDialog(self.main_window.window,
+        ask_pdf_type_dialog = Gtk.MessageDialog(self.main_window,
                     Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                       buttons = Gtk.ButtonsType.OK_CANCEL)
         ask_pdf_type_dialog.set_markup(_('What kind of PDF document do you '
@@ -643,7 +663,8 @@ class ImageReviewer_Controler:
         project_saver.serialize(project_name)
 
     def openProject(self, clear_current = True):
-        open_dialog = FileDialog('open', file_filters = [(_('OCRFeeder Projects'), [], ['*.ocrf'])])
+        open_dialog = FileDialog(self.main_window, 'open',
+                                 file_filters = [(_('OCRFeeder Projects'), [], ['*.ocrf'])])
         response = open_dialog.run()
         project_file = None
         if response == Gtk.ResponseType.OK:
@@ -663,7 +684,8 @@ class ImageReviewer_Controler:
     def __askForNumberOfPages(self, title):
         # Sync the current reviewer's page with its data
         self.__getCurrentReviewer().savePageData()
-        export_dialog = PagesToExportDialog(title)
+        export_dialog = PagesToExportDialog(self.main_window,
+                                            title)
         pages = self.pages_icon_view.getAllPages()
         # When there's only one document loaded or none,
         # we don't ask for the number of pages to export
@@ -680,7 +702,7 @@ class ImageReviewer_Controler:
             return None
 
     def __askForFileName(self, extension = ''):
-        save_dialog = FileDialog('save')
+        save_dialog = FileDialog(self.main_window, 'save')
         response = save_dialog.run()
         if response == Gtk.ResponseType.OK:
             file_name = save_dialog.get_filename()
@@ -708,7 +730,8 @@ class ImageReviewer_Controler:
     def choosePageSize(self):
         current_reviewer = self.__getCurrentReviewer()
         current_page = current_reviewer.page
-        page_size_dialog = PageSizeDialog((current_page.width, current_page.height))
+        page_size_dialog = PageSizeDialog(self.main_window,
+                                          (current_page.width, current_page.height))
         response = page_size_dialog.run()
         if response == Gtk.ResponseType.ACCEPT:
             size = page_size_dialog.getSize()
@@ -717,7 +740,7 @@ class ImageReviewer_Controler:
                     page.setSize(size)
             else:
                 current_reviewer.page.setSize(size)
-            debug('Page size: ', size)
+            debug('Page size: %s', size)
         page_size_dialog.destroy()
         self.__updateStatusBar(current_reviewer)
 
@@ -733,7 +756,8 @@ class ImageReviewer_Controler:
 
     def unpaperTool(self):
         current_reviewer = self.__getCurrentReviewer()
-        unpaper_dialog = UnpaperDialog(current_reviewer,
+        unpaper_dialog = UnpaperDialog(self.main_window,
+                                       current_reviewer,
                                     self.configuration_manager.unpaper,
                                     self.configuration_manager.TEMPORARY_FOLDER)
         if unpaper_dialog.run() == Gtk.ResponseType.ACCEPT:
@@ -797,7 +821,7 @@ class ImageReviewer_Controler:
 
     def spellCheck(self, locale):
         current_reviewer = self.__getCurrentReviewer()
-        SpellCheckerDialog(self.main_window.window, current_reviewer, locale)
+        SpellCheckerDialog(self.main_window, current_reviewer, locale)
 
     def updateOcrEngines(self, ocr_engines):
         n_pages = self.notebook.get_n_pages()
@@ -924,7 +948,7 @@ class Editor:
         if index == -1:
             return
         engine = self.ocr_engines[index][0]
-        self.box_editor.setAvailableLanguages(engine.getLanguages().keys())
+        self.box_editor.setAvailableLanguages(list(engine.getLanguages().keys()))
 
     def _onOCREngineChanged(self, combobox):
         self._resetLanguages()
@@ -934,17 +958,8 @@ class Editor:
             self.data_box.setLanguage(self.box_editor.getLanguage())
 
     def update(self, box):
-        self.box = box
-        x, y, width, height = self.data_box.updateBoundsFromBox(self.box)
-        pixbuf_width = self.pixbuf.get_width()
-        pixbuf_height = self.pixbuf.get_height()
-        sub_pixbuf = self.pixbuf.new_subpixbuf(x, y,
-                                               min(width, pixbuf_width),
-                                               min(height, pixbuf_height))
-        sub_pixbuf.x = x
-        sub_pixbuf.y = y
-        sub_pixbuf.width = pixbuf_width
-        self.data_box.setImage(sub_pixbuf)
+        self.data_box.updateBoundsFromBox(self.box)
+        self.data_box.updateImage(self.pixbuf)
 
     def updateOcrEngines(self, engines_list):
         engines_names = [engine.name for engine, path in engines_list]
@@ -965,7 +980,7 @@ class Editor:
     def performOcr(self, engine_name = None):
         selected_engine_index = self.box_editor.getSelectedOcrEngine()
         if engine_name:
-            for i in xrange(len(self.ocr_engines)):
+            for i in range(len(self.ocr_engines)):
                 if self.ocr_engines[i][0].name == engine_name:
                     selected_engine_index = i
                     break
@@ -983,7 +998,7 @@ class Editor:
     def performClassification(self, engine_name = None):
         selected_engine_index = self.box_editor.getSelectedOcrEngine()
         if engine_name:
-            for i in xrange(len(self.ocr_engines)):
+            for i in range(len(self.ocr_engines)):
                 if self.ocr_engines[i][0].name == engine_name:
                     selected_engine_index = i
                     break
@@ -995,7 +1010,7 @@ class Editor:
     def __pressedAngleDetectionButton(self, widget):
         image = graphics.convertPixbufToImage(self.box_editor.getImage())
         angle = graphics.getHorizontalAngleForText(image)
-        debug('ANGLE: ', angle)
+        debug('ANGLE: %s', angle)
         self.box_editor.setAngle(angle)
 
     def saveDataBox(self):
@@ -1031,6 +1046,9 @@ class Editor:
         self.__connectDataBoxSignals()
         self.__updateBoxColor(None, self.data_box.type)
 
+    def updateImage(self, pixbuf):
+        self.pixbuf = pixbuf
+
     def __connectDataBoxSignals(self):
         handler_id = self.data_box.connect('changed_x', self.__updateEditorX)
         self.__connected_signal_handles_list.append(handler_id)
